# -*- coding: utf-8 -*-
"""
Created on Thu Aug 24 10:15:40 2023

@author: 2905918
"""

import numpy as np
import matplotlib.pyplot as plt
import time
import sys

# Solution of u_t + a*u_{x}=0

# Final Time
T = 0.5

# Define a
a = 1

# Number of grid cells
M = 100

# length of domain [0,2] 
Ldom = 2

# Delta x
dx =  Ldom / M

# Define cell centers
x = np.arange(dx * 0.5, Ldom + 0.5 * dx, dx)
 
# Define number of time steps
NTime = 50*2*2

# Time step dt
dt = T / NTime

# Define a vector that is useful for handling the different cells
J = np.arange(0, M )  # number the cells of the domain
J1 = np.arange(1, M-1)      # the interior cells
J2 = np.arange(0, M-1)  # numbering of the cell interfaces

# Define vector for initial data
def fun_initial(x):
    v = np.zeros_like(x)
    L = np.logical_and(x >= 0.8, x <= 1.2)
    v[L] = 1.0
    return v

u0 = np.zeros(M)
u0 = fun_initial(x)

plt.figure()
plt.plot(x, u0, '-or')
plt.axis([0, Ldom, -0.1, 1.1])
plt.xlabel('x-axis')
plt.ylabel('u_0(x)')
plt.title('Solution of Linear Advection E1')
plt.grid(True)
plt.show()
#plt.hold(True)


time.sleep(2) 
print("Ready to start evolving numerical solution over time period")
#print("Press any key to continue...")
#input()  # Wait for user input (key press)
#print("Continuing with the script...")


# Define vector for solution at new t^{n+1} and old time step t^{n} 
u = np.zeros(M)
u_old = np.zeros(M)
u_exact = np.zeros(M)

# Useful quantities for the discrete scheme
lambda_1 = dt / dx

# CFL number should be <= 1 for stability
def check_CFLconstraint(value):
    if value > 1:
        print("CFL constraint not satisfied. Stopping the program.")
        sys.exit()

CFL_number = lambda_1 * a
check_CFLconstraint(CFL_number)


# Calculate the numerical solution u by going through a loop of NTime number
# of time steps

# Initial state
u_old = np.copy(u0)

for j in range(NTime):

    # Define the numerical flux at the cell-interfaces 3/2, 5/2, ... ,
    # N-1/2 = total of N-1 cell interfaces

    # Upwind
    U_half = u_old[J2]

    # Calculate solution at interior part of domain, that is, cells j=2,...,N-1
    u[J1] = u_old[J1] - a* lambda_1 * (U_half[J1] - U_half[J1 - 1])

    # Solution at cell 1
    u[0] = u0[0]

    # Solution at right boundary, cell M
    u[-1] = u0[-1]

    # Plotting
    plt.figure()
    plt.plot(x, u0, '-or')
    plt.plot(x, u, '-o')
    plt.xlabel('x')
    plt.ylabel('U(x,t)')
    plt.axis([0, Ldom, -0.1, 1.1])
    plt.title('Solution of Linear Advection E1')
    plt.grid(True)
    plt.show()
    #plt.close()

    print(f"Timestep: {j+1}") 
    #print(j+1)

    # Update "u_old" before moving forward to the next time level
    u_old = np.copy(u)
    
    #time.sleep(2)   
    

print("Computation of numerical solution finished")

# plot exact solution 

# new argument
y = x - a * T

# find index where y stays within domain [0,2]
L1 = np.logical_and(y >= 0,  y <= Ldom)

# u(x,t) = u0(x-a*T)
u_exact[L1] = fun_initial(y[L1]);


# Plotting
plt.figure()
plt.plot(x, u0, '-or')
plt.plot(x, u, '-o')
plt.plot(x[L1],u_exact[L1],'-g')
plt.xlabel('x')
plt.ylabel('U(x,t)')
plt.axis([0, Ldom, -0.1, 1.1])
plt.title('Solution of Linear Advection E1')
plt.grid(True)
plt.legend(['initial', 'numerical', 'exact'])
plt.show()




